/**
   Shade each fragment created by the rasterizer.
*/

public class PhongShader implements PipelineStage
{
   Scene scene;

   public PhongShader(Scene scene)
   {
      this.scene = scene;
   }


   /**
      Walk the Model's list of Triangle objects and for each triangle,
      walk its list of Fragment objects. For each fragment, use
      the barycentric coordinates in the fragment to interpolate
      coordinate data and normal vector data from the triangle's
      three vertices to the fragment. Use the interpolated position
      and normal to do a lighting calculation for each fragment.
   */
   public Model processModel(Model model)
   {
      Light light = scene.light;
      Matrix viewMatrix = scene.viewMatrix;

      // check if we should do smooth lighting
      if ( !model.doLighting || !model.doSmoothLighting )
      {
         return model;
      }

      if ( null == light )
      {
         System.err.println("Warning! Lighting is enabled but the Scene does not have a Light.");
         return model;
      }


      // Here is the light source as a vertex in world(!) coordinates.
      Vertex S = new Vertex(light.pos);
      // Convert light source from world coordinates to view coordinates.
      S = viewMatrix.times(S);
      // Convert the vertex to a vector.
      Vector S2 = new Vector(S.x, S.y, S.z);


      for (Triangle t : model.triangleList)
      {
         if (null == t.mtrl)
         {
            System.err.println("Warning! Lighting is enabled but a triangle does not have a Material.");
            continue;
         }

         if ( (null == t.v[0].n)
           || (null == t.v[1].n)
           || (null == t.v[2].n) )
         {
            System.err.println("Warning! Lighting is enabled but a vertex does not have a normal vector.");
            continue;
         }

         // Get all of the material, lighting, coordinate, and normal values that we need.




         // Walk this triangle's list of fragments.
         // Compute a color value at each fragment.
         for (Fragment frag : t.fragments)
         {
            // Get this fragment's barycentric coordinates and
            // interpolate the coordinates and normal vector.




            // Do the lighting calculation almost exactly like in P4_LightingSmooth.java.




         }
      }
      return model;
   }
}