/*

*/
import java.awt.Color;

public class PhongShaderClient
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", PhongShaderClient.class.getName());
         System.exit(-1);;
      }

      // Create a Material object to use with one model (brass).
      Material mtrl1 = new Material();
      mtrl1.ambient[0] = 0.33f;
      mtrl1.ambient[1] = 0.22f;
      mtrl1.ambient[2] = 0.03f;
      mtrl1.ambient[3] = 1.00f;
      mtrl1.diffuse[0] = 0.78f;
      mtrl1.diffuse[1] = 0.57f;
      mtrl1.diffuse[2] = 0.11f;
      mtrl1.diffuse[3] = 1.00f;
      mtrl1.specular[0] = 0.99f;
      mtrl1.specular[1] = 0.91f;
      mtrl1.specular[2] = 0.81f;
      mtrl1.specular[3] = 1.00f;
      mtrl1.s = 77.8f;
//    mtrl1.s = 27.8f;

      // Create a Material object to use with another model (chrome).
      Material mtrl2 = new Material();
      mtrl2.ambient[0] = 0.25f;
      mtrl2.ambient[1] = 0.25f;
      mtrl2.ambient[2] = 0.25f;
      mtrl2.ambient[3] = 1.00f;
      mtrl2.diffuse[0] = 0.4f;
      mtrl2.diffuse[1] = 0.4f;
      mtrl2.diffuse[2] = 0.4f;
      mtrl2.diffuse[3] = 1.00f;
      mtrl2.specular[0] = 0.775f;
      mtrl2.specular[1] = 0.775f;
      mtrl2.specular[2] = 0.775f;
      mtrl2.specular[3] = 1.00f;
      mtrl2.s = 76.8f;

      // Create several Model objects.
      TriangleModel m1 = new TriangleModel(3, 3, 3);
      for (Triangle t : m1.triangleList)
         t.setMaterial( mtrl2 );
      m1.doLighting = true;
      m1.doSmoothLighting = true;
      m1.doBackFaceCulling = false; // try setting this true
      m1.modelTranslate(-3, -3, -3);

      CubeModel m2 = new CubeModel(1, 1, 1);
      for (Triangle t : m2.triangleList)
         t.setMaterial( mtrl1 );
      m2.doLighting = true;
      m2.doSmoothLighting = true;
      m2.doBackFaceCulling = true;
      m2.modelTranslate(-4, 0, 0);

      SphereModel m3 = new SphereModel(k, n, 1);
      for (Triangle t : m3.triangleList)
         t.setMaterial( mtrl1 );
      m3.doLighting = true;
      m3.doSmoothLighting = true;
      m3.doBackFaceCulling = true;
      m3.modelTranslate(-2, 3, 0);

      ConeModel m4 = new ConeModel(k, n, 1, 4);
      for (Triangle t : m4.triangleList)
         t.setMaterial( mtrl1 );
      m4.doLighting = true;
      m4.doSmoothLighting = true;
      m4.doBackFaceCulling = true;
      m4.modelTranslate(0, -2, 0);

      CylinderModel m5 = new CylinderModel(k, n, 1, 4);
      for (Triangle t : m5.triangleList)
         t.setMaterial( mtrl1 );
      m5.doLighting = true;
      m5.doSmoothLighting = true;
      m5.doBackFaceCulling = true;
      m5.modelTranslate(3, 0, 0);

      Model m6 = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      m6.doLighting = false;
      m6.doBackFaceCulling = false;
      m6.modelTranslate(0, -5, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);
      scene.modelList.add(m6);

      // Create a Light object for the scene.
      Light light = new Light();
      light.pos[0] = 0.0f;
      light.pos[1] = 0.0f;
      light.pos[2] = 10.0f;
      light.pos[3] = 1.0f;
      light.ambient[0] = 1.0f;
      light.ambient[1] = 1.0f;
      light.ambient[2] = 1.0f;
      light.ambient[3] = 1.0f;
      light.diffuse[0] = 1.0f;
      light.diffuse[1] = 1.0f;
      light.diffuse[2] = 1.0f;
      light.diffuse[3] = 1.0f;
      light.specular[0] = 1.0f;
      light.specular[1] = 1.0f;
      light.specular[2] = 1.0f;
      light.specular[3] = 1.0f;
      // Add the light to the scene.
      scene.light = light;

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString() );
      //System.err.println( m1.toString(1) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);


      // Create a rendering pipeline,
      SceneRender renderer = new SceneRender(scene, fb);  // default pipeline
      // Create an instance of the shader pipeline stage.
      PipelineStage shader = new PhongShader(scene);
      // Add this pipeline stage in place of the texture fragment shader
      renderer.replacePipelineStage(12, shader);
      // Remove unneeded pipeline stages
//      renderer.removePipelineStage(12); // remove texture stage
      renderer.removePipelineStage(5);  // remove both the lighting stages
      renderer.removePipelineStage(4);
      // Check the order of the pipeline stages            .
      System.err.println( renderer );


      // Render our scene into the frame buffer.
      renderer.renderScene();
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_PhongShader_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         scene.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         renderer.renderScene();
         fb.dumpFB2File(String.format("PPM_PhongShader_Frame%02d.ppm", i));
      }
   }
}